/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "diffpressure.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define DIFFPRESSURE_DUMMY 0
#define NULL 0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void diffpressure_cfg_setup ( diffpressure_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

DIFFPRESSURE_RETVAL diffpressure_init ( diffpressure_t *ctx, diffpressure_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return DIFFPRESSURE_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, DIFFPRESSURE_DUMMY );

    return DIFFPRESSURE_OK;

}

void diffpressure_generic_transfer ( diffpressure_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);    
}

int32_t diffpressure_read_data ( diffpressure_t *ctx )
{
    spi_master_transfer_data_t block;

    uint8_t buffer[ 3 ] = { 0 };
    int32_t result = 0;

    block.tx_buffer = NULL;
    block.tx_length = 0;
    block.rx_buffer = buffer;
    block.rx_length = 3;

    spi_master_start( &ctx->spi );
    Delay_100ms( );

    diffpressure_generic_transfer( ctx, &block );

    spi_master_stop( &ctx->spi );
    
    result = buffer[ 0 ];
    result = ( result << 8 ) | buffer[ 1 ];
    result = ( result << 8 ) | buffer[ 2 ];
    
    if ( ( buffer[ 0 ] & 0x20 ) >> 5 )       
    {
        result = result - 4194304;       
    }

    return result;
}

int32_t diffpressure_get_kpa_difference ( diffpressure_t *ctx, int32_t difference )
{
    float press;
    float volt;
    int32_t result;

    volt = ( 5.0f / 2097151.0f ) * ( float ) difference;
    press = ( volt / 0.45f ) - 0.45f;
    
    result = ( int32_t ) ( press * 1000.0f );

    return result;
}

status_t diffpressure_status_check ( diffpressure_t *ctx, int32_t result )
{
    if ( !result )
    { 
        return NOT_RDY;
    }

    if ( result > 0x00BFFFFE ) 
    {
        return OVH;
    }

    if ( result > 0x009FFFFF )
    {
        return OVL;
    }
    return OK;
}

// ------------------------------------------------------------------------- END

